----------------------------------------------------------------------------------
-- Phy - transmit data out over 10baseT ethernet.
--
-- Written by hasmter@sanp.net.nz
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity Phy is
    Port ( Clk : in  STD_LOGIC;
           txp : out STD_LOGIC;
			  txm : out STD_LOGIC);
end Phy;

architecture Behavioral of Phy is
	TYPE state IS (st_idle, st_nlp, st_preamble, st_startofframe, st_data, st_crc, st_trailer, st_interframe );
	SIGNAL output 		: state;   
	SIGNAL counter 	: STD_LOGIC_VECTOR(23 downto 0) := x"FFFFFD";
	SIGNAL endOfData  : STD_LOGIC := '0';
	SIGNAL data 		: STD_LOGIC_VECTOR(7 downto 0);
	SIGNAL shiftReg 	: STD_LOGIC_VECTOR(7 downto 0);
	SIGNAL CRC			: STD_LOGIC_VECTOR(31 downto 0);
	COMPONENT PacketData
	PORT(
		Addr           : IN  STD_LOGIC_VECTOR ( 7 downto 0);          
	   IPsource       : in  STD_LOGIC_VECTOR (31 downto 0);
	   IPdestination  : in  STD_LOGIC_VECTOR (31 downto 0);
	   PhySource      : in  STD_LOGIC_VECTOR (47 downto 0);
	   PhyDestination : in  STD_LOGIC_VECTOR (47 downto 0);
		Data           : OUT STD_LOGIC_VECTOR ( 7 downto 0);
		EndOfData      : OUT STD_LOGIC
		);
	END COMPONENT;
begin
	Inst_PacketData: PacketData PORT MAP(
		Addr => counter(11 downto 4),
		IPsource       => x"99633C99",
		IPdestination  => x"99633C63",
		PhySource      => x"001234567890",
		PhyDestination => x"C80AA9657F9A",
		Data => Data,
		endOfData => endOfData
	);
	
	process (clk)
	begin
		--- Set the output signals (txp and txm)
		if clk'event and Clk='1' then
			if output = st_preamble  then
				txp <=     counter(1) xor counter(0);
				txm <= not counter(1) xor counter(0);
			elsif output = st_startofframe  then
				txp <=     counter(0);
				txm <= not counter(0);
			elsif output = st_data  then
				txp <= not shiftReg(0) xor counter(0);
				txm <=     shiftReg(0) xor counter(0);
			elsif output = st_crc then
				-- push out the CRC onto the data lines			
				txp <=     CRC(31) xor counter(0);
				txm <= not CRC(31) xor counter(0);
			elsif output = st_trailer then
				txp <= '1';
				txm <= '0';
			elsif output = st_interframe then
   			txm <= '1';
				txp <= '1';
			elsif output = st_nlp then
				txp <= '1';
				txm <= '0';
			else -- output = st_idle
				txp <= '1';
				txm <= '1';
			end if;
			
			--- Process the state
			counter <= counter+1; -- always increment, unless overridden later by state change
		   
			if output = st_preamble  then
				if counter(6 downto 0) = "1111101" then
					output <= st_startofframe;
					counter <= x"000000";
				end if;
			elsif output = st_startofframe  then
				if counter(0) = '1' then
					-- This state is only two bits long. 
					-- reset the CRC to inital value
					-- load the shift register and change state
					-- Counter is offset by 16 cycles (one byte on the air), so that 
					-- state st_data loads the second byte when this one is sent (as
					-- the address used is counter(11 downto 4))
					crc <= x"FFFFFFFF";
					counter <= x"000010";
					output <= st_data;
					shiftReg <= data;
				end if;
				
			elsif output = st_data  then
				-- set the data lines, and update the CRC
				if counter(0) = '1' then 
					if (shiftReg(0) = crc(31)) then
						crc <= crc(30 downto 0) & '0';
					else
						crc <= (crc(30 downto 0)& '0') xor x"04C11DB7";
					end if;
				end if;
								
				-- Are we at the end of the byte and need to load a new one into the shifter?
				if counter(3 downto 0) = "1111" then
					if endOfData = '1' then 
						output <= st_crc;
						counter <= x"000000";
					else
						shiftReg <= data;
					end if;
				elsif counter(0) = '1' then
               			-- Move the shift register along during the second half of a bit time
					shiftReg <= "0" & shiftReg(7 downto 1);
				end if;

			elsif output = st_crc then
				-- shift the as it is sent onto the data lines			
				if counter(0) = '1' then
					crc <= CRC(30 downto 0) & "0";
				end if;
				
				-- Have we pushed out all the CRC bits?
				if counter(5 downto 0) = "111111" then
					output <= st_trailer;
					counter <= x"000000";
				end if;	
			elsif output = st_trailer then
				-- send out three one bits (not manchester encoded)
				if counter(2 downto 1) = "11" then
					output <= st_interframe;
					counter <= x"000000";
				end if;
			elsif output = st_interframe then
				-- interframe gap of 8 bytes = 64 bits = 128 clock cycles
				if counter(7 downto 0) = "1111111" then
					output <= st_idle;
					counter <= x"000000";
				end if;
			elsif output = st_nlp then
				output <= st_idle;
			else  -- output = st_idle
				if counter = x"FFFFFF" then
					output  <= st_preamble;
				   counter <= x"000000";
				elsif counter(17 downto 0) = "11" & x"FFFF" then
					output <= st_nlp;
				end if;
			end if;
		end if;
	end process;
end Behavioral;




